/* 
Copyright (c) 2007-2008 Alloysoft, LLC. All rights reserved.
http://www.alloysoft.com
*/

var http_request;
var queue_request;
var timerID = 0;
var timerQueue = 0;
var queueID = -1;
var lastQueueEventId = 0;
var pendingRequests = new Array();
var usePersistentConnection = true;
var loggingEnabled = true;
var isMobileSafari = true;

var currentArtist = "";
var currentAlbum = "";
var currentTitle = "";
var currentRating = 0;
var currentVolume = 0;
var playingState;
var playPosition = 0;
var playlistPosition = -1;
var shuffleState = false;
var repeatState = false;
var speakers = new Array();

var currentBrowsePath = "";
var currentSearchKeywords = "";
var currentArtURL = "no_artwork.jpg";

var searchInput = "";
var searchInputTimer;

var elaspedPlayTime = 0;
var currentRequest = '';
var requestTimeout = 5000;
var isRequestPending = false;
var isQueueRequestPending = false;


// link recovery
var link_request;
var isLinkRequestPending = false;
var linkRecoveryRequests = new Array();

var timeoutRequest = 0;
var timeoutCheckQueue = 0;
var timeoutPing = 0;
var timeoutPingResponse = 0;
var timeoutVerifyPingQueue = 0;

var linkRecoveryActive = false;
var receivedPingResponse = false;
var lastEventTime = new Date();
var linkRecoveryStart = new Date();

// UI control
var currentWindowWidth = 0;
var showingMediaLib = false;
var showingPlaylist = false;
var showingSpeakers = false;
var showingSearchResults = false;
var showingMediaLibSearch = false;
var showingMediaLibActions = false;
var showingMediaLibAlphaJump = false;
var showingMediaLibSearchResults = false;
var showingSubControls = false;

var lastMonitorTime = new Date();
var itemsPerPage = 35;
var volumeIncrement = 10;
var slideSpeed = 4.0;

var mediaLibStart = 0;
var mediaLibLength = 0;
var playlistStart = 0;
var playlistLength = 0;
var showPlaylistItemOnPageLoad = false;
var browseHistory = new Array();
var currentActionItem = "";
var useMultipleSpeakers = false;

var ignoreNextVolumeUpdate = false;
var ignoreNextRatingUpdate = false;

// history
var navHistory = new Array();
var navHash = '#';
var navCurrent = '';

BrowseHistoryEntry = function()
{
	this.path = "";
	this.search = false;
	this.start = 0;
}

function createHTTPRequest() {
	var xmlHttp = null;
	
	if (window.XMLHttpRequest) {
		// Mozilla, Safari, etc.
		xmlHttp = new XMLHttpRequest();
		if (xmlHttp.overrideMimeType) {
			xmlHttp.overrideMimeType('text/xml');
		}
	} else if (window.ActiveXObject) {
		// Internet Explorer
		try {
			xmlHttp = new ActiveXObject("Msxml2.XMLHTTP");
		} catch (e) {
			try {
				xmlHttp = new ActiveXObject("Microsoft.XMLHTTP");
			} catch (e) {}
		}
	}

	return xmlHttp;
}

function sendRequest(xml)
{
	if (linkRecoveryActive) {
		if (linkRecoveryRequests.length > 0 && xml == linkRecoveryRequests[linkRecoveryRequests.length - 1]) {
			return;
		}
		
		//writeLog("DEBUG: Adding link request");
		linkRecoveryRequests.push(xml);
		return;
	}
	
	if (isRequestPending) {
		// ignore if this is the same message as the request already in progress
		if (xml == currentRequest) {
			//writeLog("DEBUG: Ignoring request, same as current in-progress");
			return;
		}
		
		//writeLog("DEBUG: Added request to pending requests: " + xml);
		pendingRequests.push(xml);
		return;
	}
	
	if (http_request && http_request.readyState != 4) {
		//writeLog("DEBUG: Aborting http_request");
		http_request.abort();
	}
	
	//writeLog("DEBUG: Sending request: " + xml);

	http_request = createHTTPRequest();

	isRequestPending = true;
	currentRequest = xml;
	//timeoutRequest = setTimeout(_cancelRequest, requestTimeout);

	// open the request, setting cache control headers so that
	// Safari does not generate cached responses on back/forward
    http_request.open('POST', '/xml', true);
	http_request.setRequestHeader("Cache-Control", "no-cache");
	http_request.setRequestHeader('If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT');
	http_request.onreadystatechange = processResponse;
    http_request.send('xml=' + xml);
}

function _cancelRequest()
{
	if (!isRequestPending || currentRequest == '') {
		return;
	}
	
	// verify that the timeout is cleared
	if (timeoutRequest) {
		clearTimeout(timeoutRequest);
		timeoutRequest = 0;
	}
	
	// abort and re-send the request
	if (http_request) {
		http_request.abort();
		http_request = null;
	}
	
	isRequestPending = false;
	//sendRequest(currentRequest);
}

function createQueue() {
	// stop the queue update timer if running
	if (timerQueue)
	{
		clearInterval(timerQueue);
		timerQueue = 0;
	}
	
	if (queue_request) {
		queue_request.abort();
		queue_request = null;
		isQueueRequestPending = false;
	}

	// ensure the queue ID is cleared - a new queue is being created
	queueID = -1;
	lastQueueEventId = 0;

	queue_request = createHTTPRequest();
	queue_request.open('POST', '/q', true);
	queue_request.setRequestHeader("Cache-Control", "no-cache");
	queue_request.setRequestHeader('If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT');
	queue_request.onreadystatechange = processQueueCreateResponse;
    queue_request.send('id=new');
}

function verifyQueueActive() {
	if (!usePersistentConnection || linkRecoveryActive || queueID < 1) {
		return;
	}
	
	// if not in link recovery mode, there should always be a queue request
	// pending, verify that this is so
	if (!queue_request || !isQueueRequestPending) {
		//writeLog("DEBUG: verifyQueueActive");
		getQueuedEvents();
	} else if (queue_request && queue_request.readyState == 0) {
		isQueueRequestPending = false;
		getQueuedEvents();
	}
}

function getQueuedEvents() {
	if (isQueueRequestPending) {
		//writeLog("DEBUG: getQueuedEvents - Queue request already pending");
		return;
	}
	
	try
	{
		if (queue_request && queue_request.readyState != 4) {
			queue_request.abort();
			queue_request = null;
			isQueueRequestPending = false;
		}
	}
	catch (e)
	{
	}
	
	if (queueID == -1) {
		//writeLog("DEBUG: getQueuedEvents - Queue ID not set");
		createQueue();
		return;
	}
	
	queue_request = createHTTPRequest();
	
	isQueueRequestPending = true;

	queue_request.open('POST', '/q', true);
	queue_request.setRequestHeader("Cache-Control", "no-cache");
	queue_request.setRequestHeader('If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT');
	queue_request.onreadystatechange = processQueueResponse;
	
	if (usePersistentConnection)
	{
		queue_request.send('id=' + queueID + '&lastEventId=' + lastQueueEventId);
	}
	else
	{
		queue_request.send('id=' + queueID);
	}
}

function processQueueCreateResponse() {
	try {
		if (queue_request.readyState != 4) {
			return;
		}

		if (queue_request.status && queue_request.status != 200) {
			// badness
			return;
		}

		queueID = parseInt(queue_request.responseText);
		lastQueueEventId = 0;
		
		// request initial state information
		try
		{
			//writeLog("DEBUG: Initial state: processQueueCreateResponse");
			requestInitialState();
		}
		catch (e)
		{
			writeLog('Error requesting initial state', e);
		}

		// start the queue update timer
		if (timerQueue) {
			clearInterval(timerQueue);
			timerQueue = 0;
		}
		
		if (usePersistentConnection)
		{
			getQueuedEvents();
			timerQueue = setInterval(verifyQueueActive, 5000);
		}
		else
		{
			timerQueue = setInterval(getQueuedEvents, 1000);
		}
	} catch (e) {
		return;
	}
}

function processQueueResponse() {
	try {
		if (queue_request.readyState != 4) {
			return;
		}
	
		isQueueRequestPending = false;

		if (queue_request.status && queue_request.status == 404) {
			// exit link recovery if necessary
			lastEventTime = new Date();
			if (linkRecoveryActive) {
				//writeLog("DEBUG: Exit: processQueueResponse 1");
				exitLinkRecovery();
			}
			
			// our queue ID has expired, create a new one
			createQueue();
			return;
		}
		else if (queue_request.status && queue_request.status != 200) {
			return;
		}
		
		if (!queue_request.status || queue_request.status != 200) {
			return;
		}
		
		if (!queue_request.responseXML) {
			return;
		}
	
		var root = queue_request.responseXML.documentElement;
		if (!root) {
			return;
		}
	
		lastEventTime = new Date();
		if (linkRecoveryActive) {
			//writeLog("DEBUG: Exit: processQueueResponse 2 (" + queue_request.status + ")");
			exitLinkRecovery();
		}
		
		// retrieve the last queue event ID, if any
		if (root.attributes)
		{
			var eventIdAttr = root.attributes.getNamedItem('lastEventId');
			if (eventIdAttr)
			{
				lastQueueEventId = parseInt(eventIdAttr.nodeValue);
				if (lastQueueEventId == NaN)
				{
					lastQueueEventId = 0;
				}
			}
		}
		
		var events = root.getElementsByTagName('event');
		for (var i=0; i < events.length; i++)
		{
			try
			{
				onMessage(events[i].firstChild);
			}
			catch (e)
			{
				var msgName = "";
				
				if (events[i] && events[i].firstChild && events[i].firstChild.tagName)
				{
					msgName = events[i].firstChild.tagName;
				}
				
				writeLog('Error processing message ' + msgName, e);
			}
		}
		
		if (usePersistentConnection)
		{
			getQueuedEvents();
		}
		
	} catch (e) {
		isQueueRequestPending = false;
		writeLog('', e);
		return;
	}
}

function onMessage(xml) {
	if (xml.tagName == 'CurrentMediaChangedEvent') {
		processCurrentMediaChangedEvent(xml);
	}
	else if (xml.tagName == 'PlayingStateChangedEvent') {
		processPlayingStateChangedEvent(xml);
	}
	else if (xml.tagName == 'VolumeChangedEvent') {
		processVolumeChangedEvent(xml);
	}
	else if (xml.tagName == 'PlaylistChangedEvent') {
		processPlaylistChangedEvent();
	}
	else if (xml.tagName == 'PlaylistPositionChangedEvent') {
		processPlaylistPositionChangedEvent(xml);
	}
	else if (xml.tagName == 'PlayPositionChangedEvent') {
		processPlayPositionChangedEvent(xml);
	}
	else if (xml.tagName == 'ShuffleStateChangedEvent') {
		processShuffleStateChangedEvent(xml);
	}
	else if (xml.tagName == 'RepeatStateChangedEvent') {
		processRepeatStateChangedEvent(xml);
	}
	else if (xml.tagName == 'MediaLibraryChangedEvent') {
		processMediaLibraryChangedEvent();
	}
	else if (xml.tagName == 'SpeakerListChangedEvent') {
		processSpeakerListChangedEvent(xml);
	}
}

function processCurrentMediaChangedEvent(xml) {
	var artist = "";
	var album = "";
	var title = "";
	var rating = 0;

	var val_nodes = xml.getElementsByTagName('title');
	if (val_nodes.length > 0)
	{
		title = getData(val_nodes[0]);
	}

	val_nodes = xml.getElementsByTagName('artist');
	if (val_nodes.length > 0)
	{
		artist = getData(val_nodes[0]);
	}

	val_nodes = xml.getElementsByTagName('album');
	if (val_nodes.length > 0)
	{
		album = getData(val_nodes[0]);
	}
	
	val_nodes = xml.getElementsByTagName('rating');
	if (val_nodes.length > 0)
	{
		rating = getDataInt(val_nodes[0]);
	}
	
	if (!ignoreNextRatingUpdate) {
		ignoreNextRatingUpdate = false;
		if (currentRating != rating) {
			currentRating = rating;
			onRatingChanged(rating);
		}
	} else {
		ignoreNextRatingUpdate = false;
	}

	if ((currentArtist != artist) || (currentAlbum != album) || (currentTitle != title))
	{
		currentArtist = artist;
		currentAlbum = album;
		currentTitle = title;

		if (document.getElementById)
		{
			var artistElem = document.getElementById('artist');
			artistElem.innerHTML = artist;
			
			var albumElem = document.getElementById('album');
			albumElem.innerHTML = album;
			
			var titleElem = document.getElementById('title');
			titleElem.innerHTML = title;
		}
		else
		{
			artist.innerHTML = artist;
			album.innerHTML = album;
			title.innerHTML = title;
		}

		sendRequest('<GetAlbumArtRequest></GetAlbumArtRequest>');
	}
}

function processPlayingStateChangedEvent(xml) {
	var val_nodes = xml.getElementsByTagName('playingState');
	if (val_nodes.length > 0)
	{
		var image = '<img src="ctrl_play.png">';
		var state = getData(val_nodes[0]);
		if (state == 'playing') {
			image = '<img src="ctrl_pause.png">';
		}
		
		if (document.getElementById)
		{
			message = document.getElementById('ctrl_play');
			message.innerHTML = image;
		}
		else
		{
			ctrl_play.innerHTML = image;
		}
	}
}

function processVolumeChangedEvent(xml) {
	if (ignoreNextVolumeUpdate) {
		ignoreNextVolumeUpdate = false;
		return;
	}
	
	var val_nodes = xml.getElementsByTagName('volume');
	if (val_nodes.length > 0)
	{
		var volume = getDataInt(val_nodes[0]);
		
		if (currentVolume != volume && volume != NaN) {
			currentVolume = volume;
			onVolumeChanged(volume);
		}
	}
}

function processPlaylistChangedEvent() {
	// refresh the current playlist view
	getPlaylistItems(playlistStart, itemsPerPage);
}

function processPlaylistPositionChangedEvent(xml) {
	var val_nodes = xml.getElementsByTagName('position');
	if (val_nodes.length > 0)
	{
		var lastPlaylistPosition = playlistPosition;
		playlistPosition = getDataInt(val_nodes[0]);
		
		var playlistItem = document.getElementById('playlist_' + lastPlaylistPosition);
		if (playlistItem) {
			playlistItem.className = "";
		}
		
		playlistItem = document.getElementById('playlist_' + playlistPosition);
		if (playlistItem) {
			playlistItem.className = "playing";
		}
		
		// if we are not showing the playlist, determine which
		// page the current media item is on and ensure it is visible
		if (!showingPlaylist) {
			if (playlistPosition < playlistStart || playlistPosition > (playlistStart + itemsPerPage)) {
				// determine which page needs to be displayed
				var page = parseInt(playlistPosition / itemsPerPage);
				getPlaylistItems(page * itemsPerPage, itemsPerPage);
			}
		}
	}
}

function processPlayPositionChangedEvent(xml) {
	var val_nodes = xml.getElementsByTagName('position');
	if (val_nodes.length > 0)
	{
		playPosition = getDataInt(val_nodes[0]);
	}
}

function processShuffleStateChangedEvent(xml) {
	var val_nodes = xml.getElementsByTagName('shuffleState');
	if (val_nodes.length > 0)
	{
		if (getData(val_nodes[0]) == 'true') {
			shuffleState = true;
		} else {
			shuffleState = false;
		}		
	}
	
	var shuffleElem = document.getElementById("shufflecontrol");
	if (shuffleElem) {
		if (shuffleState) {
			shuffleElem.style.background = 'url("shuffle_on.png") center no-repeat';
		} else {
			shuffleElem.style.background = 'url("shuffle_off.png") center no-repeat';
		}
	}
}

function processRepeatStateChangedEvent(xml) {
	var val_nodes = xml.getElementsByTagName('repeatState');
	if (val_nodes.length > 0)
	{
		if (getData(val_nodes[0]) == 'true') {
			repeatState = true;
		} else {
			repeatState = false;
		}		
	}
	
	var repeatElem = document.getElementById("repeatcontrol");
	if (repeatElem) {
		if (repeatState) {
			repeatElem.style.background = 'url("repeat_on.png") center no-repeat';
		} else {
			repeatElem.style.background = 'url("repeat_off.png") center no-repeat';
		}
	}
}

function processSpeakerListChangedEvent(xml)
{
	var speakersEnabled = false;
	
	var controlElem = document.getElementById("speakercontrolcontainer");
	var speakersElem = document.getElementById("speakerlist");
	var list = "";
	
	var result_nodes = xml.getElementsByTagName('speaker');
	for (var i=0; i < result_nodes.length; i++)
	{
		var result = result_nodes[i];

		var val_nodes = result.getElementsByTagName('name');
		if (val_nodes.length > 0)
		{
			var name = getData(val_nodes[0]);
			
			var val_nodes = result.getElementsByTagName('id');
			if (val_nodes.length > 0)
			{
				var id = getDataInt(val_nodes[0]);
				list += '<li onclick="handleSpeakerSelection(' + id + ')">' + name + '</li>';
			}
		}
	}
	
	speakersElem.innerHTML = list;
	
	if (result_nodes.length > 1)
	{
		controlElem.style.display = 'block';
	}
	else
	{
		controlElem.style.display = 'none';
	}
}

function processMediaLibraryChangedEvent() {
	// refresh the current media library view
	browseLibrary(currentBrowsePath, 0, itemsPerPage);
}

function processResponse()
{
	try
	{	
		if (http_request.readyState != 4) {
			return;
		}

		if (timeoutRequest) {
			clearTimeout(timeoutRequest);
			timeoutRequest = 0;
		}
		
		currentRequest = '';
		isRequestPending = false;

		if (!http_request.status || http_request.status != 200) {
			return;
		}

		if (!http_request.responseXML) {
			return;
		}
		
		var root = http_request.responseXML.documentElement;
		if (!root) {
			return;
		}
		
		try
		{
			if (root.tagName == 'GetCurrentMediaResponse') {
				// equivalent to CurrentMediaChangedEvent
				processCurrentMediaChangedEvent(root);
			} else if (root.tagName == 'GetPlayingStateResponse') {
				// equivalent to PlayingStateChangedEvent
				processPlayingStateChangedEvent(root);
			} else if (root.tagName == 'GetVolumeResponse') {
				// equivalent to VolumeChangedEvent
				processVolumeChangedEvent(root);
			} else if (root.tagName == 'GetShuffleStateResponse') {
				// equivalent to ShuffleStateChangedEvent
				processShuffleStateChangedEvent(root);
			} else if (root.tagName == 'GetRepeatStateResponse') {
				// equivalent to RepeatStateChangedEvent
				processRepeatStateChangedEvent(root);
			} else if (root.tagName == 'GetAlbumArtResponse') {
				processGetAlbumArtResponse(root);
			} else if (root.tagName == 'MediaSearchResponse') {
				processMediaSearchResponse(root);
			} else if (root.tagName == 'BrowseLibraryResponse') {
				processBrowseLibraryResponse(root);
			} else if (root.tagName == 'GetPlaylistItemsResponse') {
				processGetPlaylistItemsResponse(root);
			} else if (root.tagName == 'GetPlaylistPositionResponse') {
				// equivalent to PlaylistPositionChangedEvent
				processPlaylistPositionChangedEvent(root);
			} else if (root.tagName == 'GetSpeakerListResponse') {
				// equivalent to SpeakerListChangedEvent
				processSpeakerListChangedEvent(root);
			} else {
				// common error handler
				processResponseForErrors(root);
			}
		}
		catch (e)
		{
			var msgName = "";
			
			if (root.tagName)
			{
				msgName = root.tagName;
			}
			
			writeLog('Error processing response ' + msgName, e);
		}
		
		// send the next pending request, if any
		if (pendingRequests.length > 0) {
			sendRequest(pendingRequests.pop());
		}
	} catch (e) {
		writeLog('', e);
		return;
	}
}

function processResponseForErrors(xml) {
	var val_nodes = xml.getElementsByTagName('returnCode');
	if (val_nodes.length > 0)
	{
		if (val_nodes[0].firstChild)
		{
			var returnCode = getDataInt(val_nodes[0]);
			
			if (returnCode == 106)
			{
				alert('Please check "Enable access for assistive devices" on your Mac to use this feature.');
			}
		}
	}
}

function processGetAlbumArtResponse(xml) {
	var artURL = "";

	var val_nodes = xml.getElementsByTagName('artURL');
	if (val_nodes.length > 0)
	{
		if (val_nodes[0].firstChild)
		{
			artURL = getData(val_nodes[0]);
		}
	}
	
	if (artURL != currentArtURL) {
		currentArtURL = artURL;
		if (onAlbumArtChanged) {
			onAlbumArtChanged(currentArtURL);
		}
	}
}

function processBrowseLibraryResponse(xml) {
	var list = "<ul>";
	var result;
	var type = "";
	var imgFile = "";
	var isFolder = false;
	var showAddOption = false;
	var alphaStartResult = -1;
	
	if (showingMediaLibAlphaJump)
		showMediaLibAlphaJump();
	
	if (showingMediaLibSearch) {
		showMediaLibSearch();
	}
	
	if (showingMediaLibSearchResults) {
		showingMediaLibSearchResults = false;
	}
	
	if (showingMediaLib) {
		hideAddressBar();
	}
	
	showingSearchResults = false;

	var val_nodes = xml.getElementsByTagName('path');
	if (val_nodes.length > 0)
	{
		// TODO: firstChild is null, investigate
		if (val_nodes[0].firstChild) {
			currentBrowsePath = getData(val_nodes[0]);	
		} else {
			currentBrowsePath = "";
		}
	}
	
	// identify the name of the current path
	var name = 'Media Library';
	if (currentBrowsePath.length > 1) {
		var pos = currentBrowsePath.lastIndexOf('\\');
		if (pos != -1) {
			pos++;
			name = currentBrowsePath.substr(pos, currentBrowsePath.length - pos);
			if (name.length > 0) {			
				if (name.charAt(name.length - 1) == '\\') {
					name = name.substr(0, name.length - 1);
				}
			} else {
				name = 'Media Library';
			}
		}
	}
	
	if (document.getElementById)
	{
		message = document.getElementById('libheadertitle');
		message.innerHTML = name;
	}
	else
	{
		libheadertitle.innerHTML = name;
	}
	
	// retrieve the new start position
	val_nodes = xml.getElementsByTagName('start');
	if (val_nodes.length > 0)
	{
		mediaLibStart = getDataInt(val_nodes[0]);
	}
	
	// retrieve the total count if included
	val_nodes = xml.getElementsByTagName('totalCount');
	if (val_nodes.length > 0)
	{
		mediaLibLength = getDataInt(val_nodes[0]);
	}
	
	// retrieve the alpha start position included
	val_nodes = xml.getElementsByTagName('alphaStartResult');
	if (val_nodes.length > 0)
	{
		alphaStartResult = getDataInt(val_nodes[0]);
	}
	
	// add the page controls if necessary
	if (mediaLibLength > itemsPerPage) {
		list += '<li>';
		
		if (mediaLibStart > 0) {
			list += '<div id="navprev" onclick="mediaLibPrevPage()">&lt; Prev</div>';
		} else {
			list += '<div id="navprev">&nbsp;</div>';
		}
		
		list += '<div id="navalphajump" onclick="showMediaLibAlphaJump()">A-Z</div>';
		
		if ((mediaLibStart + itemsPerPage) < mediaLibLength) {
			list += '<div id="navnext" onclick="mediaLibNextPage()">Next &gt;</div>';
		} else {
			list += '<div id="navnext">&nbsp;</div>';
		}
		
		list += '</li>';
	}

	var result_nodes = xml.getElementsByTagName('result');
	for (var i=0; i < result_nodes.length; i++)
	{
		result = result_nodes[i];
		var itemId = null;
		isFolder = false;
		showAddOption = true;

		val_nodes = result.getElementsByTagName('type');
		if (val_nodes.length > 0)
		{
			type = getData(val_nodes[0]);
			if (type == 'folder') {
				isFolder = true;
			}
		}

		val_nodes = result.getElementsByTagName('id');
		if (val_nodes.length > 0)
		{
			itemId = getData(val_nodes[0]);
		}

		val_nodes = result.getElementsByTagName('name');
		if (val_nodes.length > 0)
		{
			var name = getData(val_nodes[0]);
			
			if (isFolder) {
				if (name == 'Music') {
					imgFile = 'cd';
				} else if (name == 'Videos') {
					imgFile = 'video';
				} else if (name == 'TV') {
					imgFile = 'tv';
				} else if (name == 'Genres') {
					imgFile = 'genre';
				} else if (name == 'Artists') {
					imgFile = 'artist';
				} else if (name == 'Albums') {
					imgFile = 'album';
				} else if (name == 'Playlists') {
					imgFile = 'playlist';
				} else if (name == 'Tracks') {
					imgFile = 'music';
				} else if (name == 'Podcasts') {
					imgFile = 'podcast';
				} else if (name == 'Radio') {
					imgFile = 'podcast';
				} else {
					imgFile = 'music';
				}
			} else {
				if (type == 'genre') {
					imgFile = 'genre';
				} else 	if (type == 'artist') {
					imgFile = 'artist';
				} else 	if (type == 'album') {
					imgFile = 'album';
				} else if (type == 'podcast show') {
					imgFile = 'podcast';
				} else if (type == 'tv series') {
					imgFile = 'artist';
					showAddOption = false;
				} else 	if (type == 'playlist') {
					imgFile = 'playlist';
				} else 	if (type == 'playlist folder' || type == 'radio folder') {
					imgFile = 'playlist_folder';
					showAddOption = false;
				} else 	if (type == 'info') {
					imgFile = 'info';
				} else {
					imgFile = 'music';
				}
			}

			var newPath = currentBrowsePath + "\\" + name;
			newPath = newPath.replace(/\\/g, '\\\\');
			newPath = newPath.replace(/\'/g, "\\'");
			newPath = newPath.replace(/\"/g, '&quot;');
			newPath = "'" + newPath + "'";

			list += '<li id="medialib_' + (mediaLibStart + i) + '" ';
			
			if (!itemId) {
				if (!isFolder && showAddOption) {
					list += 'class="group" '
				}
			} else if (type == "music" || type == "podcast") {
				list += '><div id="grouptitle"';
			}
			
			if (itemId) {
				list += 'onclick="handlePlayItem(this, ' + itemId + ')"';
			} else {
				list += 'onclick="handleBrowseLibrary(this, ' + newPath + ')"';
			}
			
			list += '>';
			
			if (imgFile != '') {
			 list += '<img src="' + imgFile + '.png">';
			}
			
			list += name;
			
			if (!itemId) {
				if (!isFolder && showAddOption) {
					list += '<div id="showactions" onclick="showMediaLibActions(' + newPath + ')"></div>';
				}
			} else if (type == "music" || type == "podcast") {
				list += '</div><div id="showactions" onclick="handleSingleItemAdd(this, ' + itemId + ')"></div>';
			}
			
			list += '</li>';
		}
	}
	
	// add the page controls if necessary
	if (mediaLibLength > itemsPerPage) {
		list += '<li>';
		
		if (mediaLibStart > 0) {
			list += '<div id="navprev" onclick="mediaLibPrevPage()">&lt; Prev</div>';
		} else {
			list += '<div id="navprev">&nbsp;</div>';
		}
		
		list += '<div id="navalphajump" onclick="showMediaLibAlphaJump()">A-Z</div>';
		
		if ((mediaLibStart + itemsPerPage) < mediaLibLength) {
			list += '<div id="navnext" onclick="mediaLibNextPage()">Next &gt;</div>';
		} else {
			list += '<div id="navnext">&nbsp;</div>';
		}
		
		list += '</li>';
	}

	list += "</ul>";

	if (document.getElementById)
	{
		message = document.getElementById('libcontents');
		message.innerHTML = list;
	}
	
	// if we have received an alpha start position in response to a
	// browse starting from a particular letter, scroll to the result
	// if it is beyond the initial screen
	if (alphaStartResult != -1 && showingMediaLib)
	{
		var mediaLibItem = document.getElementById('medialib_' + alphaStartResult);
		if (mediaLibItem) {
			slideTo(mediaLibItem.offsetTop);
		}
	}
}

function processMediaSearchResponse(xml) {
	var list = "<ul>";
	var result;
	
	// backup scrolling to work around Safari displaying address bar
	// when the keyboard is shown
	hideAddressBar();
	
	if (!showingMediaLibSearchResults)
	{
		addBrowseHistoryEntry();
		showingMediaLibSearchResults = true;
		
		if (document.getElementById)
		{
			message = document.getElementById('libheadertitle');
			message.innerHTML = 'Search';
		}
		else
		{
			libheadertitle.innerHTML = 'Search';
		}
	}
	
	// retrieve the new start position
	val_nodes = xml.getElementsByTagName('start');
	if (val_nodes.length > 0)
	{
		mediaLibStart = getDataInt(val_nodes[0]);
	}
	else
	{
		mediaLibStart = 0;
	}
	
	// retrieve the total count if included
	val_nodes = xml.getElementsByTagName('totalCount');
	if (val_nodes.length > 0)
	{
		mediaLibLength = getDataInt(val_nodes[0]);
	}
	
	var result_nodes = xml.getElementsByTagName('result');
	for (var i=0; i < result_nodes.length; i++)
	{
		result = result_nodes[i];
		var type = "";
		var itemId = null;
		var itemClass = "music";
		var imgFile = "music";
		var newPath = "";

		var val_nodes = result.getElementsByTagName('type');
		if (val_nodes.length > 0)
		{
			type = getData(val_nodes[0]);
			itemClass = type;
			
			if (itemClass == 'podcast show') {
				itemClass = 'podcast';
			} else if (itemClass == 'podcast') {
				itemClass = 'music';
			}
		}

		val_nodes = result.getElementsByTagName('id');
		if (val_nodes.length > 0)
		{
			itemId = getData(val_nodes[0]);
		}

		val_nodes = result.getElementsByTagName('name');
		if (val_nodes.length > 0)
		{
			var name = getData(val_nodes[0]);
			
			list += '<li';

			if (itemId)
			{
				if (type != 'playlist') {
					list += '><div id="grouptitle" onclick="handlePlayItem(this, ' + itemId + ')"';
				} else {
					list += ' onclick="handlePlayItem(this, ' + itemId + ')"';
				}
			}
			else
			{
				newPath = "\\Music\\";

				if (type == "genre")
					newPath += "Genres\\";
				else if (type == "artist")
					newPath += "Artists\\";
				else if (type == "album")
					newPath += "Albums\\";
				else if (type == "podcast show")
					newPath = "\\Podcasts\\";

				newPath += name;
				newPath = newPath.replace(/\\/g, '\\\\');
				newPath = newPath.replace(/\'/g, "\\'");
				newPath = newPath.replace(/\"/g, '&quot;');
				newPath = "'" + newPath + "'";
				
				list += ' class="group" onclick="handleBrowseLibrary(this, ' + newPath + ')"';
			}
			
			list += '><img src="' + itemClass + '.png">' + name;
			
			if (!itemId) {
				list += '<div id="showactions" onclick="showMediaLibActions(' + newPath + ')"></div>';
			} else {
				if (type != 'playlist') {
					list += '</div><div id="showactions" onclick="handleSingleItemAdd(this, ' + itemId + ')"></div>';
				}
			}
			
			list += '</li>';
		}
	}

	list += "</ul>";

	if (document.getElementById)
	{
		message = document.getElementById('libcontents');
		message.innerHTML = list;
	}
	else
	{
		libcontents.innerHTML = list;
	}
	
	showingSearchResults = true;
}

function processGetPlaylistItemsResponse(xml) {
	var itemIndex = 0;
	var list = '<ul>';
	
	if (showingPlaylist) {
		hideAddressBar();
	}
	
	var val_nodes = xml.getElementsByTagName('start');
	if (val_nodes.length > 0)
	{
		playlistStart = getDataInt(val_nodes[0]);
	}
	else
	{
		playlistStart = 0;
	}
	
	val_nodes = xml.getElementsByTagName('totalCount');
	if (val_nodes.length > 0)
	{
		playlistLength = getDataInt(val_nodes[0]);
	}
	
	// add the page controls if necessary
	if (playlistLength > itemsPerPage) {
		list += '<li>';
		
		if (playlistStart > 0) {
			list += '<div id="navprev" onclick="playlistPrevPage()">&lt; Prev</div>';
		}
		
		if ((playlistStart + itemsPerPage) < playlistLength) {
			list += '<div id="navnext" onclick="playlistNextPage()">Next &gt;</div>';
		}
		
		list += '</li>';
	}

	val_nodes = xml.getElementsByTagName('item');
	for (i=0; i < val_nodes.length; i++)
	{
		itemIndex = playlistStart + i;
		list += '<li id="playlist_' + itemIndex + '" onclick="handlePlaylistJump(this, ' + itemIndex + ')"';
		
		if (itemIndex == playlistPosition) {
			list += 'class="playing" ';
		}
		
		var artist = '';
		var title = '';
		var pos = getData(val_nodes[i]).indexOf(' - ');
		if (pos != -1)
		{
			artist = getData(val_nodes[i]).substr(0, pos);
			
			pos += 3;
			title = getData(val_nodes[i]).substr(pos, getData(val_nodes[i]).length - pos);
		}
		else
		{
			title = getData(val_nodes[i]);
		}
		
		//list += '>' + title + '<br><span class="listArtist">' + artist + '</span></li>';
		list += '>' + title + ' <span class="listArtist">' + artist + '</span></li>';
	}
	
	if (val_nodes.length < 1) {
		list += '<li>No items</li>';
	}
	
	// add the page controls if necessary
	if (playlistLength > itemsPerPage) {
		list += '<li>';
		
		if (playlistStart > 0) {
			list += '<div id="navprev" onclick="playlistPrevPage()">&lt; Prev</div>';
		}
		
		if ((playlistStart + itemsPerPage) < playlistLength) {
			list += '<div id="navnext" onclick="playlistNextPage()">Next &gt;</div>';
		}
		
		list += '</li>';
	}
	
	list += '</ul>';

	if (document.getElementById)
	{
		message = document.getElementById('playlistcontents');
		message.innerHTML = list;
	}
	else
	{
		playlist.innerHTML = list;
	}
	
	// show the currently playing item, if requested
	if (showPlaylistItemOnPageLoad) {
		showPlaylistItemOnPageLoad = false
		
		if (playlistPosition >= playlistStart && playlistPosition < (playlistStart + itemsPerPage)) {
			showCurrentPlaylistItem();
		}
	}
}

function playpause()
{
	sendRequest('<PlayPauseRequest></PlayPauseRequest>');
}

function play()
{
	sendRequest('<PlayRequest></PlayRequest>');
}

function pause()
{
	sendRequest('<PauseRequest></PauseRequest>');
}

function stop()
{
	sendRequest('<StopRequest></StopRequest>');
}

function prev()
{
	sendRequest('<PreviousItemRequest></PreviousItemRequest>');
}

function next()
{
	sendRequest('<NextItemRequest></NextItemRequest>');
}

function playItem(item)
{
	if (isNaN(Number(item))) {
		// a media library URL was specified
		sendRequest('<PlayMediaRequest><libPath>' + escape(item) + '</libPath></PlayMediaRequest>');
	} else {
		// a media library ID was specified
		sendRequest('<PlayMediaRequest><id>' + item + '</id></PlayMediaRequest>');
	}	
}

function playlistAdd(item)
{
	if (isNaN(Number(item))) {
		// a media library URL was specified
		sendRequest('<PlaylistAddRequest><libPath>' + escape(item) + '</libPath></PlaylistAddRequest>');
	} else {
		// a media library ID was specified
		sendRequest('<PlaylistAddRequest><id>' + item + '</id></PlaylistAddRequest>');
	}	
}

function setPlaylistPosition(index)
{
	sendRequest('<SetPlaylistPositionRequest><position>' + index + '</position></SetPlaylistPositionRequest>');
}

function refreshState()
{
	sendRequest('<GetCurrentMediaRequest></GetCurrentMediaRequest>');
}

function browseLibrary(browsePath, start, count, includeTotalCount)
{
	start = (start == null) ? 0 : start;
	count = (count == null) ? -1 : count;
	includeTotalCount = (includeTotalCount == null) ? true : includeTotalCount;

	var request;
	if (isNaN(Number(start)))
	{
		// browsing to the first entry for a specific letter was requested
		request = '<BrowseLibraryRequest><start>' + start + '</start><count>' + count + '</count><path>' + escape(browsePath) + '</path>';
	}
	else
	{
		if (start < 0) {
			start = 0;
		}
		
		// a standard browse based on a start index was requested
		request = '<BrowseLibraryRequest><start>' + start + '</start><count>' + count + '</count><path>' + escape(browsePath) + '</path>';
	}
	
	if (includeTotalCount) {
		request += '<includeTotalCount>true</includeTotalCount>';
	}
	
	request += '</BrowseLibraryRequest>';
	sendRequest(request);
}

function searchLibrary(searchTerms, start, count)
{
	start = (start == null) ? 0 : start;
	count = (count == null) ? -1 : count;
	
	if (start < 0) {
		start = 0;
	}
	
	sendRequest('<MediaSearchRequest><start>' + start + '</start><count>' + count + '</count><any>' + escape(searchTerms) + '</any></MediaSearchRequest>');
}

function getPlaylistItems(start, count, includeTotalCount)
{
	start = (start == null) ? 0 : start;
	count = (count == null) ? -1 : count;
	includeTotalCount = (includeTotalCount == null) ? true : includeTotalCount;
	
	var request = '<GetPlaylistItemsRequest><start>' + start + '</start><count>' + count + '</count>';
	
	if (includeTotalCount) {
		request += '<includeTotalCount>true</includeTotalCount>';
	}
	
	request += '</GetPlaylistItemsRequest>';
	
	sendRequest(request);
}

function toggleShuffleState() {
	sendRequest('<SetShuffleStateRequest><shuffleState>' + !shuffleState + '</shuffleState></SetShuffleStateRequest>');
}

function toggleRepeatState() {
	sendRequest('<SetRepeatStateRequest><repeatState>' + !repeatState + '</repeatState></SetRepeatStateRequest>');
}

function setVolume(volume) {
	if (volume < 0) {
		volume = 0;
	} else if (volume > 100) {
		volume = 100;
	}
	
	ignoreNextVolumeUpdate = true;
	currentVolume = volume;
	sendRequest('<SetVolumeRequest><volume>' + volume + '</volume></SetVolumeRequest>');
}

function volumeUp() {
	setVolume(currentVolume + volumeIncrement);
	onVolumeChanged(currentVolume);
}

function volumeDown() {
	setVolume(currentVolume - volumeIncrement);
	onVolumeChanged(currentVolume);
}

function setRating(rating) {
	if (rating < 0) {
		rating = 0;
	} else if (rating > 100) {
		rating = 100;
	}
	
	sendRequest('<SetTrackRatingRequest><rating>' + rating + '</rating></SetTrackRatingRequest>');
}

function selectSpeaker(id) {
	sendRequest('<SelectSpeakerRequest><id>' + id + '</id></SelectSpeakerRequest>');
}

function enableSpeaker(id, enabled) {
	sendRequest('<EnableSpeakerRequest><id>' + id + '</id><enabled>' + enabled + '</enabled></EnableSpeakerRequest>');
}

function requestInitialState()
{
	sendRequest('<GetPlayingStateRequest></GetPlayingStateRequest>');
	sendRequest('<GetCurrentMediaRequest></GetCurrentMediaRequest>');
	sendRequest('<GetVolumeRequest></GetVolumeRequest>');
	sendRequest('<GetShuffleStateRequest></GetShuffleStateRequest>');
	sendRequest('<GetRepeatStateRequest></GetRepeatStateRequest>');
	sendRequest('<GetPlaylistPositionRequest></GetPlaylistPositionRequest>');
	sendRequest('<GetSpeakerListRequest></GetSpeakerListRequest>');
	browseLibrary(currentBrowsePath, mediaLibStart, itemsPerPage);
	getPlaylistItems(0, itemsPerPage);
}

function testForMobileSafari()
{
	try
	{
		if (navigator.userAgent)
		{
			var pos = navigator.userAgent.indexOf(' Mobile/');
			if (pos != -1)
			{
				pos = navigator.userAgent.indexOf(' Safari/');
				if (pos != -1)
				{
					return true;
				}
			}
		}
	}
	catch (e) {}
	
	return false;
}

function onLoad()
{
	isMobileSafari = testForMobileSafari();
	
	// ensure that we display Now Playing despite any initial hash on load
	if (isMobileSafari && location.hash != navCurrent && location.hash != 'NowPlaying')
	{
		navCurrent = 'NowPlaying';
		location.hash = navCurrent;
	}
	
	// immediately apply any appropriate UI modification
	uiMonitor();
	
	setTimeout(setupApp, 100);
}

function setupApp() {
	hideAddressBar();
	createQueue();
	setInterval(sleepMonitor, 1000);
	setInterval(uiMonitor, 300);
	
	// read cookie settings
	var val = getCookie("useMultipleSpeakers");
	if (val != null && val == "true" && !useMultipleSpeakers) {
		toggleUseMultipleSpeakers();
	}
}

function hideAddressBar() {
	window.scrollTo(0,1);
}

function backToNowPlaying() {
	if (isMobileSafari)
	{
		// use browser history to navigate
		if (navHistory.length > 0)
		{
			history.back();
		}
		else
		{
			setLocation('', false);
		}
	}		
	else
	{
		// just hide the appropriate interface
		if (showingMediaLib)
		{
			showMediaLib(false);
		}
		else if (showingPlaylist)
		{
			showPlaylist(false);
		}
		else if (showingSpeakers)
		{
			showSpeakers(false);
		}
	}
}

function showMediaLib(recordHistory) {
	if (document.getElementById)
	{
		hideAddressBar();
		//browseHistory.splice(0, browseHistory.length);
		
		var medialib = document.getElementById('medialib');
		var container = document.getElementById('container');
		
		if (!showingMediaLib) {
			
			/*if ((currentBrowsePath != "" && currentBrowsePath != "\\") || (showingSearchResults)) {
				browseLibrary("", 0, itemsPerPage);
			}*/
			
			setLocation('Media', recordHistory);
			
			if (showingPlaylist) {
				showPlaylist();
			}
			
			showingMediaLib = true;
			medialib.style.overflow = 'visible';
			medialib.style.visibility = 'visible';
			container.style.overflow = 'visible';
		} else {
			if (showingMediaLibSearch) {
				showMediaLibSearch();
			}
			
			if (showingMediaLibAlphaJump)
				showMediaLibAlphaJump();
					
			showingMediaLib = false;
			medialib.style.visibility = 'hidden';
			medialib.style.overflow = 'hidden';
			container.style.overflow = 'hidden';
		}
	}
}

function showPlaylist(recordHistory) {
	if (document.getElementById)
	{		
		var playlist = document.getElementById('playlist');
		var container = document.getElementById('container');
		
		if (!showingPlaylist) {
			
			setLocation('Playlist', recordHistory);
			
			if (showingMediaLib) {
				showMediaLib(false);
			}
			
			hideAddressBar();
			playlist.style.overflow = 'visible';
			playlist.style.visibility = 'visible';
			container.style.overflow = 'visible';
			showingPlaylist = true;
		} else {			
			playlist.style.visibility = 'hidden';
			playlist.style.overflow = 'hidden';
			container.style.overflow = 'hidden';
			showingPlaylist = false;
			hideAddressBar();
		}
	}
}

function showSpeakers(recordHistory) {
	if (document.getElementById)
	{		
		var speakers = document.getElementById('speakers');
		var container = document.getElementById('container');
		
		if (!showingSpeakers) {
			
			setLocation('Speakers', recordHistory);
			
			hideAddressBar();
			speakers.style.overflow = 'visible';
			speakers.style.visibility = 'visible';
			container.style.overflow = 'visible';
			showingSpeakers = true;
		} else {			
			speakers.style.visibility = 'hidden';
			speakers.style.overflow = 'hidden';
			container.style.overflow = 'hidden';
			showingSpeakers = false;
			hideAddressBar();
		}
	}
}

function showMediaLibSearch() {
	
	if (showingMediaLibAlphaJump)
		showMediaLibAlphaJump();
	
	if (document.getElementById)
	{
		var searchHeader = document.getElementById('libsearchfield');
		
		if (!showingMediaLibSearch) {
			searchHeader.style.visibility = 'visible';
			
			var searchField = document.getElementById('searchInput');
			if (searchField) {
				searchField.focus();
			}
			
			searchInputTimer = setInterval(searchInputMonitor, 300);
			
			showingMediaLibSearch = true;
		} else {
			searchHeader.style.visibility = 'hidden';
			
			var searchField = document.getElementById('searchInput');
			searchField.value = '';
			
			if (searchInputTimer) {
				clearInterval(searchInputTimer);
				searchInputTimer = null;
			}
			
			showingMediaLibSearch = false;
		}
	}
}

function showMediaLibActions(path) {
	if (document.getElementById)
	{		
		var itemActions = document.getElementById('itemactions');
		currentActionItem = path;
		
		if (!showingMediaLibActions) {
			hideAddressBar();
			itemActions.style.visibility = 'visible';
			showingMediaLibActions = true;
		} else {
			itemActions.style.visibility = 'hidden';
			showingMediaLibActions = false;
			hideAddressBar();
		}
	}
}

function showMediaLibAlphaJump() {
	if (document.getElementById)
	{		
		var alphaJumpElem = document.getElementById('alphajump');
		
		if (!showingMediaLibAlphaJump) {
			hideAddressBar();
			alphaJumpElem.style.visibility = 'visible';
			showingMediaLibAlphaJump = true;
		} else {
			alphaJumpElem.style.visibility = 'hidden';
			showingMediaLibAlphaJump = false;
			hideAddressBar();
		}
	}
}

function showCurrentPlaylistItem() {
	// first ensure that the page containing this item is loaded
	if (playlistPosition < playlistStart || playlistPosition >= (playlistStart + itemsPerPage)) {
		// determine which page needs to be displayed
		var page = parseInt(playlistPosition / itemsPerPage);
		getPlaylistItems(page * itemsPerPage, itemsPerPage);
		showPlaylistItemOnPageLoad = true;
		return;
	}
	
	// show the playlist if it is not already visible
	// delay the scroll request so scrolling works properly
	if (!showingPlaylist){
		showPlaylist();
		setTimeout(showCurrentPlaylistItem, 100);
		return;
	}
	
	var playlistItem = document.getElementById('playlist_' + playlistPosition);
	if (playlistItem) {
		window.scrollTo(0, playlistItem.offsetTop);
	}
}

function addBrowseHistoryEntry()
{
	var entry = new BrowseHistoryEntry();
	entry.start = mediaLibStart;
	entry.search = showingMediaLibSearchResults;
	if (!entry.search)
	{
		entry.path = currentBrowsePath;
	}
	else
	{
		entry.path = currentSearchKeywords;
	}
	
	browseHistory.push(entry);
}

function navigateLibBack() {
	if (browseHistory.length < 1) {
		backToNowPlaying();
		return;
	}
	
	var entry = browseHistory.pop();
	if (!entry.search)
	{
		browseLibrary(entry.path, entry.start, itemsPerPage);
	}
	else
	{
		if (!showingMediaLibSearchResults)
		{
			showingMediaLibSearchResults = true;

			if (document.getElementById)
			{
				message = document.getElementById('libheadertitle');
				message.innerHTML = 'Search';
			}
			else
			{
				libheadertitle.innerHTML = 'Search';
			}
		}
		
		currentSearchKeywords = entry.path;
		searchLibrary(entry.path, entry.start, itemsPerPage);
	}
}

function onAlbumArtChanged(url) {
	if (url == "") {
		url = "no_artwork.jpg";
	}
	
	var artHTML = "";
	if (url != "") {
		artHTML = '<img id="artwork" src="' + url + '">';
	}
	
	if (document.getElementById)
	{
		var albumart = document.getElementById("albumart");
		albumart.innerHTML = artHTML;
	}
	else
	{
		albumart.innerHTML = artHTML;
	}
}

function onRatingChanged(rating) {
	var numStars = rating / 20;
	
	for (i=1; i <= 5; i++) {
		var starElem = document.getElementById("star" + i);
		if (starElem) {
			if (i <= numStars) {
				starElem.className = "starred";
			} else {
				starElem.className = "";
			}
		}
	}
}

function onVolumeChanged(volume) {
	var volumeBarBack = document.getElementById('volumecontrol');
	var volumeBar = document.getElementById('volumecontrol_on');
	if (volumeBar && volumeBarBack) {
		// calculate the width we need
		var width = (volumeBarBack.offsetWidth - 20);
		width *= (currentVolume / 100.0);
		volumeBar.style.width = width + "px";
	}	
}

function searchInput_keyUp()
{
	// ignore if not in search mode
	if (!showingMediaLibSearch) {
		return;
	}
	
	var searchField = document.getElementById('searchInput');

	if (searchField.value.length > 2)
	{
		currentSearchKeywords = searchField.value;
		searchLibrary(searchField.value, 0, itemsPerPage);
	}
	else
	{
		var results = document.getElementById('libcontents');
		results.innerHTML = "";
	}
}

function toggleControls() {
	var subcontrols = document.getElementById('subcontrols');
	
	hideAddressBar();
	
	if (!showingSubControls) {
		subcontrols.style.visibility = 'visible';
		showingSubControls = true;
	} else {
		subcontrols.style.visibility = 'hidden';
		showingSubControls = false;
	}
}

function handleBrowseLibrary(object, path) {
	// object.className = "on";
	hideAddressBar();
	
	if (showingMediaLib) {
		addBrowseHistoryEntry();
	}
	
	browseLibrary(path, 0, itemsPerPage);
}

function handlePlayItem(object, itemId) {
	backToNowPlaying();
	playItem(itemId);
}

function handlePlaylistJump(object, itemId) {
	backToNowPlaying();
	setPlaylistPosition(itemId);
}

function handleRating(rating) {
	currentRating = rating;
	//ignoreNextRatingUpdate = true;
	onRatingChanged(rating);
	setRating(rating);
}

function handleActionPlay() {
	playItem(currentActionItem);
	showMediaLibActions();
}

function handleActionAdd() {
	playlistAdd(currentActionItem);
	showMediaLibActions();
}

function handleSingleItemAdd(object, itemId) {
	object.className = "off";
	playlistAdd(itemId);
}

function handleSpeakerSelection(id) {
	if (!useMultipleSpeakers) {
		backToNowPlaying();
		selectSpeaker(id);	
	} else {
		enableSpeaker(id, true);
	}
}

function toggleUseMultipleSpeakers() {
	useMultipleSpeakers = !useMultipleSpeakers;
	
	var msgElem = document.getElementById('speakermsg');
	var btnElem = document.getElementById('multiplespeakerbtn');
	btnElem.setAttribute("enabled", useMultipleSpeakers);
	
	if (useMultipleSpeakers) {
		msgElem.innerHTML = "Tap on a speaker name to toggle its audio on or off.";
	} else {
		msgElem.innerHTML = "Tap on a speaker name to send all audio to that speaker.";
	}
	
	if (useMultipleSpeakers) {
		setCookie("useMultipleSpeakers", "true", 365);
	} else {
		deleteCookie("useMultipleSpeakers");
	}
}

function mediaLibPrevPage() {
	browseLibrary(currentBrowsePath, mediaLibStart - itemsPerPage, itemsPerPage, false);
}

function mediaLibNextPage() {
	browseLibrary(currentBrowsePath, mediaLibStart + itemsPerPage, itemsPerPage, false);
}

function mediaLibJumpToLetter(letter) {
	browseLibrary(currentBrowsePath, letter, itemsPerPage, false);
	
	if (showingMediaLibAlphaJump)
		showMediaLibAlphaJump();
}

function playlistPrevPage() {
	getPlaylistItems(playlistStart - itemsPerPage, itemsPerPage, false);
}

function playlistNextPage() {
	getPlaylistItems(playlistStart + itemsPerPage, itemsPerPage, false);
}

function adjustVolume(event) {
	var posX = event.offsetX?(event.offsetX):event.pageX-document.getElementById("volumecontrol").offsetLeft;
	
	var volumeBarBack = document.getElementById('volumecontrol');
	var volumeBar = document.getElementById('volumecontrol_on');
	if (volumeBar && volumeBarBack) {
		// calculate the new volume, accepting a buffer zone to mute or set max volume
		var newVolume;
		
		if (posX > 10 && posX < volumeBarBack.offsetWidth - 10) {
			newVolume = posX / (volumeBarBack.offsetWidth - 20);
			newVolume *= 100.0;
			newVolume = parseInt(newVolume);
		} else if (posX < 10) {
			newVolume = 0;
		} else {
			newVolume = 100;
		}
		
		setVolume(newVolume);
		
		// go ahead and move the volume bar now so the user receives feedback
		onVolumeChanged(currentVolume);
	}
}

function setLocation(page, recordHistory)
{
	if (!isMobileSafari)
		return;
		
	if (recordHistory == null || recordHistory == true)
		navHistory.push(location.hash);
		
	navCurrent = page;
	location.hash = navCurrent;
}

function slideTo(pos)
{
	setTimeout(function()
    {
		var percent = 100;
        var slideTimer = setInterval(slide, 0);

        function slide()
        {
            percent -= slideSpeed;
            if (percent <= 0)
            {
                percent = 0;
                clearInterval(slideTimer);
            }

            var x = ((100-percent)/100) * pos;
            scrollTo(0, x);
        }
    }, 0);
}

function setCookie(name, value, days) {
	var expires = "";
	
	if (days) {
		var date = new Date();
		date.setTime(date.getTime()+(days*24*60*60*1000));
		expires = "; expires="+date.toGMTString();
	}
	
	document.cookie = name + "=" + value + expires + "; path=/";
}

function getCookie(name) {
	name += "=";
	var cookieArray = document.cookie.split(';');
	for(var i=0; i < cookieArray.length; i++) {
		var testCookie = cookieArray[i];
		
		while (testCookie.charAt(0)==' ') {
			testCookie = testCookie.substring(1, testCookie.length);
		}
		
		if (testCookie.indexOf(name) == 0)
			return testCookie.substring(name.length, testCookie.length);
	}
	return null;
}

function deleteCookie(name) {
	setCookie(name, "", -1);
}

/** Monitors the search input field for changes to support
search-as-you-type on the iPhone. Normally this would be
accomplished via an onkeyup handler, but the iPhone does
not support this in firmware 1.0 */
function searchInputMonitor() {
	if (!showingMediaLibSearch) {
		return;
	}
	
	var searchField = document.getElementById('searchInput');
	if (searchField.value != searchInput) {
		searchInput = searchField.value;
		
		if (searchInput.length > 2)
		{
			currentSearchKeywords = searchInput;
			searchLibrary(searchInput, 0, itemsPerPage);
		}
		else
		{
			var results = document.getElementById('libcontents');
			results.innerHTML = "";
		}
	}
}

function uiMonitor()
{
	// only perform this UI processing for Mobile Safari
	if (!isMobileSafari)
		return;
		
	if (currentWindowWidth != window.innerWidth)
	{
		currentWindowWidth = window.innerWidth;
		var orientation = (currentWindowWidth == 320) ? "portrait" : "landscape";
		document.body.setAttribute("orientation", orientation);
		
		// ensure that the address bar is hidden if showing the now playing screen
		if (!showingPlaylist && !showingMediaLib)
		{
			hideAddressBar();
		}
	}
	
	if (location.hash != navCurrent)
	{
		// user pressed the back or forward button
		var page = location.hash.substr(navHash.length)
		if (page == navCurrent)
		{
			// browser uses a different representation for the same location, we can ignore it
			navCurrent = location.hash;
			verifyQueueActive();
			return;
		}
		
		navCurrent = location.hash;
		
		// check to see if the user has navigated backwards
		if (navHistory.length > 0 && location.hash == navHistory[navHistory.length - 1])
		{
			navHistory.pop();
		}
		else
		{
			// the user has navigated to a different destinaton, using the forward button
			// or manual hash entry
			navHistory.push(location.hash);
		}
				
		if (page == 'Media')
		{
			if (!showingMediaLib)
				showMediaLib(false);
		}
		else if (page == 'Playlist')
		{
			if (!showingPlaylist)
				showPlaylist(false);
		}
		else if (page == 'Speakers')
		{
			if (!showingSpeakers)
				showSpeakers(false);
		}
		else
		{			
			if (showingMediaLib)
			{
				showMediaLib(false);
			}
			else if (showingPlaylist)
			{
				showPlaylist(false);
			}
			else if (showingSpeakers)
			{
				showSpeakers(false);
			}
		}
		
		verifyQueueActive();
	}
}

/** Monitors for pauses in the application's JavaScript execution,
such as when the device enters sleep mode. When the device wakes,
ping the server to ensure our link with it is still active. In some 
cases the phone will take a few seconds to restore Wi-Fi, during which 
time HTTP requests will be made through EDGE and take a long time to
fail. This prevents events from being received from the server, so the
ping function is used to identify when our connection is restored and
re-activate the queue. 

1. When a period of idle time has been detected, enter link recovery
2. Wait x seconds. If a queue event was received, no further action needed
3. Ping the server
4. Once a successful ping is received, monitor for a queue event (the server will trigger one as part of the ping response)
5. If no queue event is received after x seconds, abort the queue connection and re-create

*/
function sleepMonitor() {
	var currentTime = new Date();
	
	if ((currentTime - lastMonitorTime > 7000) && (!linkRecoveryActive))
	{
		// begin link recovery procedures to ensure our persistent
		// queue connection is maintained
		enterLinkRecovery();
	}
	
	lastMonitorTime = currentTime;
}

function enterLinkRecovery() {
	// exit immediately if already in link recovery mode
	if (linkRecoveryActive) {
		return;
	}
	
	linkRecoveryRequests.splice(0, linkRecoveryRequests.length);
		
	// ensure that link recovery has been properly cleaned up
	exitLinkRecovery();
	
	// suspend the queue timer if active
	if (timerQueue)
	{
		clearInterval(timerQueue);
		timerQueue = 0;
	}
	
	// cancel any in-progress request
	if (http_request) {
		http_request.abort();
		http_request = null;
	}
	
	isRequestPending = false;
	pendingRequests.splice(0, pendingRequests.length);
	
	linkRecoveryActive = true;
	//writeLog("DEBUG: Link recovery active");
	linkRecoveryStart = new Date();
	timeoutCheckQueue = setTimeout(checkQueue, 2000);
}

function exitLinkRecovery() {
	
	if (timeoutCheckQueue) {
		clearTimeout(timeoutCheckQueue);
		timeoutCheckQueue = 0;
	}
	
	if (timeoutPing) {
		clearTimeout(timeoutPing);
		timeoutPing = 0;
	}
	
	if (timeoutPingResponse) {
		clearTimeout(timeoutPingResponse);
		timeoutPingResponse = 0;
	}
	
	if (timeoutVerifyPingQueue) {
		clearTimeout(timeoutVerifyPingQueue);
		timeoutVerifyPingQueue = 0;
	}
	
	// restore the queue timer if necessary
	if (timerQueue == 0)
	{
		if (usePersistentConnection)
		{
			timerQueue = setInterval(verifyQueueActive, 5000);
		}
		else
		{
			timerQueue = setInterval(getQueuedEvents, 1000);
		}
	}
	
	if (link_request && link_request.readyState != 4) {
		link_request.abort();
		link_request = null;
	}
	
	isLinkRequestPending = false;
	linkRecoveryActive = false;
	//writeLog("DEBUG: Link recovery exited");
	
	// send any requests queued up while in link recovery
	while (linkRecoveryRequests.length > 0) {
		sendRequest(linkRecoveryRequests.pop());
	}
}

function checkQueue() {
	if (lastEventTime <= linkRecoveryStart) {
		// we did not receive an event, ping the server
		pingServer();
	} else {
		// no further action needed
		//writeLog("DEBUG: Exit: checkQueue");
		exitLinkRecovery();
	}
}

/** Pings the server to ensure the queue link is still active */
function pingServer() {
	// cancel any pending ping request
	if (isLinkRequestPending && link_request) {
		link_request.abort();
	}
	
	link_request = createHTTPRequest();
	
	isLinkRequestPending = true;
	receivedPingResponse = false;

	link_request.open('POST', '/q', true);
	link_request.setRequestHeader("Cache-Control", "no-cache");
	link_request.setRequestHeader('If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT');
	link_request.onreadystatechange = processLinkResponse;
	link_request.send('action=ping&id='+queueID);
	
	timeoutPingResponse = setTimeout(verifyPingResponse, 1000);
}

function verifyPingResponse() {
	if (!receivedPingResponse) {
		if (queue_request) {
			queue_request.abort();
			queue_request = null;
			isQueueRequestPending = false;
		}
		
		// keep pinging the server until a response is received
		pingServer();
	}
}

function processLinkResponse() {
	try {
		if (link_request.readyState != 4) {
			return;
		}
		
		isLinkRequestPending = false;

		if (!link_request.status || link_request.status != 200) {
			// badness
			return;
		}
		
		if (!link_request.responseText) {
			return;
		}

		if (link_request.responseText == 'ok') {
			// we have received a successful ping response			
			if (timeoutPingResponse) {
				clearTimeout(timeoutPingResponse);
				timeoutPingResponse = 0;
			}
			
			receivedPingResponse = true;
			
			// if we are still in link recovery mode, wait to see if the queue receives the ping event
			if (linkRecoveryActive) {
				if (timeoutVerifyPingQueue) {
					clearTimeout(timeoutVerifyPingQueue);
				}
				
				timeoutVerifyPingQueue = setTimeout(verifyQueuePing, 2000);
			}
			
			getQueuedEvents();
		} else if (link_request.responseText == 'not found') {
			// the queue no longer exists
			if (linkRecoveryActive) {
				//writeLog("DEBUG: Exit: processLinkResponse");
				exitLinkRecovery();
			}
			
			// our queue ID has expired, create a new one
			createQueue();
		}
		
	} catch (e) {
		isLinkRequestPending = false;
		writeLog('', e);
		return;
	}
}

function verifyQueuePing() {
	// link recovery can be exited at this point
	//writeLog("DEBUG: Exit: verifyQueuePing");
	exitLinkRecovery();
	
	if (lastEventTime <= linkRecoveryStart) {
		// we did not receive an event, abort the current queue connection
		if (queue_request) {
			queue_request.abort();
			queue_request = null;
			isQueueRequestPending = false;
		}
		
		// this queue does not appear to be responding, create a new one
		//writeLog("DEBUG: verifyQueuePing: Creating new queue");
		createQueue();
	}
}

function getData(elem)
{
	if (elem.firstChild) {
		return elem.firstChild.data;
	} else {
		return '';
	}
}

function getDataInt(elem)
{
	if (elem.firstChild) {
		return parseInt(elem.firstChild.data);
	} else {
		return -1;
	}
}

function writeLog(msg, e)
{
	if (!loggingEnabled)
	{
		return;
	}
	
	try
	{
		if (e)
		{
			if (e.lineNumber)
			{
				msg += ': ' + e.toString() + ' at line ' + e.lineNumber + ' of ' + e.fileName + '\nstack:\n' + e.stack;
			}
			else
			{
				msg += ': ' + e.toString();
			}
		}
		
		console.log(msg);
	}
	catch (loge)
	{		
	}
}
