/* 
Copyright (c) 2007 Alloysoft, LLC. All rights reserved.
http://www.alloysoft.com
*/

MediaItem = function()
{
	this.artist = "";
	this.album = "";
	this.title = "";
	this.rating = 0;
}

RemotePlayer = function()
{
	// Public properties /////////////////////////////////////////////////
	
	this.currentMedia = new MediaItem();
	this.volume = 0;
	this.playingState = "stopped";
	this.playlistPosition = 0;
	this.shuffleState = false;
	this.repeatState = false;
	this.currentArtURL = "";
	
	this.volumeIncrement = 10;
	
	// event handlers
	this.onCurrentMediaChanged = null;
	this.onAlbumArtChanged = null;
	this.onVolumeChanged = null;
	this.onPlayingStateChanged = null;
	this.onShuffleStateChanged = null;
	this.onRepeatStateChanged = null;
	this.onPlaylistChanged = null;
	
	// Private variables /////////////////////////////////////////////////
	
	var self = this;
	var http_request;
	var queue_request;
	var timerID = 0;
	var timerQueue = 0;
	var queueID = -1;
	var isRequestPending = false;
	var isQueueRequestPending = false;
	var pendingRequests = new Array();
	
	// link recovery
	var link_request;
	var isLinkRequestPending = false;

	var timeoutCheckQueue = 0;
	var timeoutPing = 0;
	var timeoutPingResponse = 0;
	var timeoutVerifyPingQueue = 0;

	var linkRecoveryActive = false;
	var receivedPingResponse = false;
	var lastEventTime = new Date();
	var linkRecoveryStart = new Date();
	
	// Privledged methods ////////////////////////////////////////////////
	
	this.start = function()
	{
		createQueue();
	}
	
	this.playpause = function()
	{
		sendRequest('<PlayPauseRequest></PlayPauseRequest>');
	}

	this.play = function()
	{
		sendRequest('<PlayRequest></PlayRequest>');
	}

	this.pause = function()
	{
		sendRequest('<PauseRequest></PauseRequest>');
	}

	this.stop = function()
	{
		sendRequest('<StopRequest></StopRequest>');
	}

	this.prev = function()
	{
		sendRequest('<PreviousItemRequest></PreviousItemRequest>');
	}

	this.next = function()
	{
		sendRequest('<NextItemRequest></NextItemRequest>');
	}
	
	this.toggleShuffle = function() 
	{
		sendRequest('<SetShuffleStateRequest><shuffleState>' + !shuffleState + '</shuffleState></SetShuffleStateRequest>');
	}

	this.toggleRepeat = function() 
	{
		sendRequest('<SetRepeatStateRequest><repeatState>' + !repeatState + '</repeatState></SetRepeatStateRequest>');
	}

	this.setVolume = function(volume) 
	{
		if (volume < 0)
		{
			volume = 0;
		}
		else if (volume > 100)
		{
			volume = 100;
		}

		this.volume = volume;
		sendRequest('<SetVolumeRequest><volume>' + volume + '</volume></SetVolumeRequest>');
	}
	
	this.setRating = function(rating) 
	{
		if (rating < 0)
		{
			rating = 0;
		}
		else if (rating > 100)
		{
			rating = 100;
		}

		sendRequest('<SetTrackRatingRequest><rating>' + rating + '</rating></SetTrackRatingRequest>');
	}

	this.volumeUp = function()
	{
		setVolume(this.currentVolume + this.volumeIncrement);
		
		if (onVolumeChanged)
			onVolumeChanged(this.currentVolume);
	}

	this.volumeDown = function()
	{
		setVolume(this.currentVolume - this.volumeIncrement);
		
		if (onVolumeChanged)
			onVolumeChanged(this.currentVolume);
	}
	
	// Private methods ///////////////////////////////////////////////////
	
	function createHTTPRequest()
	{
		var xmlHttp = null;

		if (window.XMLHttpRequest)
		{
			// Mozilla, Safari, etc.
			xmlHttp = new XMLHttpRequest();
			if (xmlHttp.overrideMimeType)
			{
				xmlHttp.overrideMimeType('text/xml');
			}
		} 
		else if (window.ActiveXObject)
		{
			// Internet Explorer
			try {
				xmlHttp = new ActiveXObject("Msxml2.XMLHTTP");
			} catch (e) {
				try {
					xmlHttp = new ActiveXObject("Microsoft.XMLHTTP");
				} catch (e) {}
			}
		}

		return xmlHttp;
	}
	
	function sendRequest(xml)
	{
		if (isRequestPending)
		{
			pendingRequests.push(xml);
			return;
		}

		http_request = createHTTPRequest();

		isRequestPending = true;

		http_request.onreadystatechange = processResponse;
	    http_request.open('POST', '/xml', true);
	    http_request.send('xml=' + xml);
	}
	
	function createQueue()
	{
		// stop the queue update timer if running
		if (timerQueue)
		{
			clearInterval(timerQueue);
			timerQueue = 0;
		}

		// ensure the queue ID is cleared - a new queue is being created
		queueID = -1;

		queue_request = createHTTPRequest();
		queue_request.onreadystatechange = processQueueCreateResponse;
		queue_request.open('POST', '/q', true);
	    queue_request.send('id=new');
	}

	function getQueuedEvents()
	{
		if (isQueueRequestPending)
		{
			return;
		}

		if (queue_request)
		{
			queue_request.abort();
		}

		queue_request = createHTTPRequest();
		queue_request.onreadystatechange = processQueueResponse;

		if (queueID == -1)
		{
			createQueue();
			return;
		}

		isQueueRequestPending = true;

		queue_request.open('POST', '/q', true);
		queue_request.send('id=' + queueID);
	}

	function processQueueCreateResponse()
	{
		try {
			if (queue_request.readyState != 4)
			{
				return;
			}

			if (queue_request.status != 200)
			{
				// badness
				return;
			}

			queueID = parseInt(queue_request.responseText);

			// request initial state information
			requestInitialState();

			// start the queue update timer
			if (timerQueue)
			{
				clearInterval(timerQueue);
			}

			getQueuedEvents();
			timerQueue = setInterval(getQueuedEvents, 1000);
		} catch (e) {
			return;
		}
	}

	function processQueueResponse()
	{
		//try {
			if (queue_request.readyState != 4)
			{
				return;
			}

			isQueueRequestPending = false;
			
			if (!queue_request.status)
				return;

			if (queue_request.status == 404)
			{
				// exit link recovery if necessary
				lastEventTime = new Date();
				if (linkRecoveryActive)
				{
					exitLinkRecovery();
				}

				// our queue ID has expired, create a new one
				createQueue();
				return;
			}
			else if (queue_request.status != 200)
			{
				return;
			}
			
			if (!queue_request.responseXML)
				return;

			var root = queue_request.responseXML.documentElement;
			if (!root) 
			{
				return;
			}

			lastEventTime = new Date();
			if (linkRecoveryActive)
			{
				exitLinkRecovery();
			}

			var events = root.getElementsByTagName('event');
			for (var i=0; i < events.length; i++)
			{
				onEvent(events[i].firstChild);
			}
			
			getQueuedEvents();
		/*} catch (e) {
			isQueueRequestPending = false;
			return;
		}*/
	}
	
	function processResponse()
	{
		//try {
			if (http_request.readyState != 4)
			{
				return;
			}

			isRequestPending = false;

			if (http_request.status != 200)
			{
				return;
			}

			var xmlDoc = http_request.responseXML;
			var root = xmlDoc.documentElement;
			if (!root)
			{
				return;
			}

			onResponse(root);

			// send the next pending request, if any
			if (pendingRequests.length > 0)
			{
				sendRequest(pendingRequests.pop());
			}
		/*} catch (e) {
			return;
		}*/
	}

	function onEvent(xml)
	{
		if (xml.tagName == 'CurrentMediaChangedEvent')
		{
			processCurrentMediaChangedEvent(xml);
		}
		else if (xml.tagName == 'PlayingStateChangedEvent')
		{
			processPlayingStateChangedEvent(xml);
		}
		else if (xml.tagName == 'VolumeChangedEvent')
		{
			processVolumeChangedEvent(xml);
		}
		else if (xml.tagName == 'PlaylistChangedEvent')
		{
			processPlaylistChangedEvent();
		}
		else if (xml.tagName == 'PlaylistPositionChangedEvent')
		{
			processPlaylistPositionChangedEvent(xml);
		}
		else if (xml.tagName == 'PlayPositionChangedEvent')
		{
			processPlayPositionChangedEvent(xml);
		}
		else if (xml.tagName == 'ShuffleStateChangedEvent') 
		{
			processShuffleStateChangedEvent(xml);
		}
		else if (xml.tagName == 'RepeatStateChangedEvent')
		{
			processRepeatStateChangedEvent(xml);
		}
		else if (xml.tagName == 'MediaLibraryChangedEvent') 
		{
			processMediaLibraryChangedEvent();
		}
	}
	
	function onResponse(xml) 
	{
		if (xml.tagName == 'GetCurrentMediaResponse')
		{
			// equivalent to CurrentMediaChangedEvent
			processCurrentMediaChangedEvent(xml);
		} 
		else if (xml.tagName == 'GetPlayingStateResponse')
		{
			// equivalent to PlayingStateChangedEvent
			processPlayingStateChangedEvent(xml);
		}
		else if (xml.tagName == 'GetVolumeResponse')
		{
			// equivalent to VolumeChangedEvent
			processVolumeChangedEvent(xml);
		}
		else if (xml.tagName == 'GetShuffleStateResponse')
		{
			// equivalent to ShuffleStateChangedEvent
			processShuffleStateChangedEvent(xml);
		}
		else if (xml.tagName == 'GetRepeatStateResponse')
		{
			// equivalent to RepeatStateChangedEvent
			processRepeatStateChangedEvent(xml);
		}
		else if (xml.tagName == 'GetPlaylistPositionResponse')
		{
			// equivalent to PlaylistPositionChangedEvent
			processPlaylistPositionChangedEvent(xml);
		}
		else if (xml.tagName == 'GetAlbumArtResponse')
		{
			processGetAlbumArtResponse(xml);
		}
		else if (xml.tagName == 'MediaSearchResponse') 
		{
			processMediaSearchResponse(xml);
		}
		else if (xml.tagName == 'BrowseLibraryResponse')
		{
			processBrowseLibraryResponse(xml);
		}
		else if (xml.tagName == 'GetPlaylistItemsResponse')
		{
			processGetPlaylistItemsResponse(xml);
		}
	}
	
	function processCurrentMediaChangedEvent(xml) 
	{
		var newMediaItem = new MediaItem();

		var val_nodes = xml.getElementsByTagName('title');
		if (val_nodes.length > 0)
		{
			newMediaItem.title = val_nodes[0].firstChild.data;
		}

		val_nodes = xml.getElementsByTagName('artist');
		if (val_nodes.length > 0)
		{
			newMediaItem.artist = val_nodes[0].firstChild.data;
		}

		val_nodes = xml.getElementsByTagName('album');
		if (val_nodes.length > 0)
		{
			newMediaItem.album = val_nodes[0].firstChild.data;
		}

		val_nodes = xml.getElementsByTagName('rating');
		if (val_nodes.length > 0)
		{
			newMediaItem.rating = parseInt(val_nodes[0].firstChild.data);
		}

		/*var newRating = parseInt(val_nodes[0].firstChild.data);
		if (self.currentMedia.rating != newRating) {
			if (!ignoreNextRatingUpdate) {
				newMediaItem.rating = newRating;
				
				if (self.onRatingChanged) {
					self.onRatingChanged(newRating);
				}
			} else {
				ignoreNextRatingUpdate = false;
			}
		}*/

		if (self.currentMedia != newMediaItem)
		{
			self.currentMedia = newMediaItem;

			if (onCurrentMediaChanged)
			{
				onCurrentMediaChanged(newMediaItem);
			}

			sendRequest('<GetAlbumArtRequest></GetAlbumArtRequest>');
		}
	}
	
	function processPlayingStateChangedEvent(xml)
	{
		var val_nodes = xml.getElementsByTagName('playingState');
		if (val_nodes.length > 0)
		{
			var state = val_nodes[0].firstChild.data;
			if (self.playingState != state)
			{
				self.playingState = state;
				if (self.onPlayingStateChanged)
				{
					self.onPlayingStateChanged(state);
				}
			}
		}
	}

	function processVolumeChangedEvent(xml)
	{
		/*if (ignoreNextVolumeUpdate) {
			ignoreNextVolumeUpdate = false;
			return;
		}*/

		var val_nodes = xml.getElementsByTagName('volume');
		if (val_nodes.length > 0)
		{
			var volume = parseInt(val_nodes[0].firstChild.data);
			if (self.volume != volume) 
			{
				self.volume = volume;
				if (self.onVolumeChanged)
				{
					self.onVolumeChanged(volume);
				}
			}
		}
	}

	function processPlaylistChangedEvent()
	{
		// refresh the current playlist view
		//getPlaylistItems(playlistStart, itemsPerPage);
	}

	function processPlaylistPositionChangedEvent(xml)
	{
		var val_nodes = xml.getElementsByTagName('position');
		if (val_nodes.length > 0)
		{
			self.playlistPosition = parseInt(val_nodes[0].firstChild.data);
			
			if (self.onPlaylistPositionChanged)
			{
				self.onPlaylistPositionChanged(self.playlistPosition);
			}
		}
	}

	function processPlayPositionChangedEvent(xml)
	{
		var val_nodes = xml.getElementsByTagName('position');
		if (val_nodes.length > 0)
		{
			self.playPosition = parseInt(val_nodes[0].firstChild.data);
		}
	}

	function processShuffleStateChangedEvent(xml) {
		var val_nodes = xml.getElementsByTagName('shuffleState');
		if (val_nodes.length > 0)
		{
			if (val_nodes[0].firstChild.data == 'true') 
			{
				self.shuffleState = true;
			}
			else
			{
				self.shuffleState = false;
			}		
		}

		if (self.onShuffleStateChanged) 
		{
			self.onShuffleStateChanged(self.shuffleState);
		}
	}

	function processRepeatStateChangedEvent(xml) {
		var val_nodes = xml.getElementsByTagName('repeatState');
		if (val_nodes.length > 0)
		{
			if (val_nodes[0].firstChild.data == 'true')
			{
				self.repeatState = true;
			} 
			else 
			{
				self.repeatState = false;
			}		
		}

		if (self.onRepeatStateChanged)
		{
			self.onRepeatStateChanged(self.repeatState);
		}
	}

	function processMediaLibraryChangedEvent()
	{
		// refresh the current media library view
		//browseLibrary(currentBrowsePath, 0, itemsPerPage);
	}
	
	function processGetAlbumArtResponse(xml)
	{
		var artURL = "";

		var val_nodes = xml.getElementsByTagName('artURL');
		if (val_nodes.length > 0)
		{
			if (val_nodes[0].firstChild)
			{
				artURL = val_nodes[0].firstChild.data;
			}
		}

		if (self.currentArtURL != artURL)
		{
			self.currentArtURL = artURL;
			
			if (self.onAlbumArtChanged)
			{
				self.onAlbumArtChanged(artURL);
			}
		}
	}
	
	function requestInitialState()
	{
		sendRequest('<GetPlayingStateRequest></GetPlayingStateRequest>');
		sendRequest('<GetVolumeRequest></GetVolumeRequest>');
		sendRequest('<GetShuffleStateRequest></GetShuffleStateRequest>');
		sendRequest('<GetRepeatStateRequest></GetRepeatStateRequest>');
		sendRequest('<GetCurrentMediaRequest></GetCurrentMediaRequest>');
		sendRequest('<GetPlaylistPositionRequest></GetPlaylistPositionRequest>');
		//browseLibrary(currentBrowsePath, mediaLibStart, itemsPerPage);
		//getPlaylistItems(0, itemsPerPage);
	}
}